﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;   //必须添加，不然DllImport报错
namespace ai_oneshot
{
    class Program
    {
        static void Main(string[] args)
        {
            int temp;
            // 设备索引，默认设备0，如果使用一个设备，默认索引是0，如果是多个设备，则需要执行ReadSN函数来确定多个设备的索引
            int DevIndex = 0;
            //初始化一个空数组 用来存放采集的数据 数组长度要大于等于Getai函数获取的数据*开启的通道数
            float[] Ai = new float[80000];

            // 调用函数时，返回错误代码
            // 打开设备
            temp = USB3000DLL.USB3OpenDevice(DevIndex);
            //返回值以及代表SN码 以及型号信息
            temp = USB3000DLL.USB3ReadDevcieSN(DevIndex);
            Console.WriteLine("SN码：" + temp);
            temp = USB3000DLL.USB3ReadDevcieModel(DevIndex);
            Console.WriteLine("型号为USB-" + temp);
            // 设置 Conv Out 端口输出信号源为模拟输入采样时钟。如果不使用该信号，可不设置。
            temp = USB3000DLL.SetUSB3ExtConvOutSource(DevIndex, 2);

            // 设置 Ext Trig Out 端口输出信号源为模拟输入触发信号。如果不使用该信号，可不设置。
            temp = USB3000DLL.SetUSB3ExtTrigOutSource(DevIndex, 0);
            // 选中前 8 个通道，进行通道关闭。采集卡默认开启8通道，开启设备后，先进行默认通道关闭
            // 防止没有关闭，导致后面开启通道混乱导致数据无法与对应需要开启的通道数据对应上

            for (byte i = 0; i < 8; i++)
            {
                // 选中第 i 个通道 。0为关闭 1位开启
                temp = USB3000DLL.SetUSB3AiChanSel(DevIndex, i, 0);

            }
            
             // 选中第 0 个通道
            temp = USB3000DLL.SetUSB3AiChanSel(DevIndex, 0, 1);
            // 设置第 0 个通道的量程，设置10代表正负10v量程，设置5代表正负5v量程
            temp = USB3000DLL.SetUSB3AiRange(DevIndex, 0, 10);
            
            // 设置为模拟量oneshot采集模式 即获取到固定的点数采集卡不再采集
            temp = USB3000DLL.SetUSB3AiSampleMode(DevIndex, 1);

            // 设置每一次 OneShot 采集点数为每通道 1000 点
            temp = USB3000DLL.SetUSB3AiOneShotPoints(DevIndex, 100);

            // 设置采样周期为 10000ns，即 100kSa/s 根据 f = 1/T 进行换算   单通道采样率为 总采样率/开启通道数
            temp = USB3000DLL.SetUSB3AiSampleRate(DevIndex, 10000);

            // 设置模拟输入采集使用 AiSoftTrig 软件触发
            temp = USB3000DLL.SetUSB3AiTrigSource(DevIndex, 0);

            // 设置模拟输入采样时钟源为 Ai 内部采样时钟源。该时钟源频率由 SetUSB3AiSampleRate() 函数设定。
            temp = USB3000DLL.SetUSB3AiConvSource(DevIndex, 0);

            // 设置 Ai 预触发点数为 0 如果无预触需求 可不执行此函数
            temp = USB3000DLL.SetUSB3AiPreTrigPoints(DevIndex, 0);

            // 清空 AiFifo
            temp = USB3000DLL.SetUSB3ClrAiFifo(DevIndex);

            // 启动 AiSoftTrig 软件触发
            temp = USB3000DLL.SetUSB3AiSoftTrig(DevIndex);

            // 开始读取数据。
            // 如果需要长时间连续采集，持续循环使用 USB3GetAi() 函数读取即可。
            // GetAi函数返回值代表缓冲区还剩余多少个数据个数，每通道缓冲区可缓冲2百万个数据点 
            
            // 每通道读取 100 个点存放于 Ai 数组中，超时时间为 1000ms
            // 数据存放规则为前 100 个数据是 Ai 0 通道；
            //          接下来 100 个数据是 Ai 1 通道；
            //          接下来 100 个数据是 Ai 2 通道；
            // 依此类推
            temp = USB3000DLL.USB3GetAi(DevIndex, 100, Ai, 1000);
            Console.WriteLine("缓冲区剩余个数：" + temp);
            //仅打印第一个通道每次获取的100个数据的第一个数据作为演示
            Console.WriteLine(Ai[0]);
            

            // 清除 Ai 触发信号
            temp = USB3000DLL.SetUSB3ClrAiTrigger(DevIndex);
            // 清空 AiFifo
            temp = USB3000DLL.SetUSB3ClrAiFifo(DevIndex);
            // 关闭设备
            temp = USB3000DLL.USB3CloseDevice(DevIndex);

            /*如需了解更多函数参数意义，则需要访问smmacq.cn对应产品系列函数手册进行查询
             *以及了解各类函数返回值负数代表的错误意义            
             */
        }
    }
    class USB3000DLL
    {
        [DllImport("USB3000.dll")]
        public static extern int FindUSB3DAQ();
        [DllImport("USB3000.dll")]
        public static extern int USB3OpenDevice(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int USB3CloseDevice(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int USB3ReadDevcieSN(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int USB3ReadDevcieModel(int DevIndex);

        //--------------------------------------------------------------------------
        // Ananlog Input Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiSampleMode(int DevIndex, byte AiSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiConnectType(int DevIndex, byte AiConnectType);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiRange(int DevIndex, byte Chan, float AiRange);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiChanSel(int DevIndex, byte Chan, byte Sel);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiTrigSource(int DevIndex, byte AiTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiConvSource(int DevIndex, byte AiConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiPreTrigPoints(int DevIndex, uint AiPreTrigPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiOneShotPoints(int DevIndex, uint AiOneShotPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAiFifo(int DevIndex);

        //--------------------------------------------------------------------------
        // Digital I/O Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiSampleMode(int DevIndex, byte DiSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiTrigSource(int DevIndex, byte DiTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiConvSource(int DevIndex, byte DiConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiPreTrigPoints(int DevIndex, uint DiPreTrigPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiOneShotPoints(int DevIndex, uint DiOneShotPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDiFifo(int DevIndex);

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoSampleMode(int DevIndex, byte DoSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoTrigSource(int DevIndex, byte DoTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoConvSource(int DevIndex, byte DoConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoCycle(int DevIndex, uint DoCycle);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoDataFifo(int DevIndex, uint[] Value, uint Len);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDoFifo(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoWaveCtrl(int DevIndex, uint Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoImmediately(int DevIndex, uint Chan, uint Value);

        //--------------------------------------------------------------------------
        // Ananlog Output Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSampleRate(int DevIndex, byte Chan, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSampleMode(int DevIndex, byte Chan, byte AoSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoTrigSource(int DevIndex, byte Chan, byte AoTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoConvSource(int DevIndex, byte Chan, byte AoConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoCycle(int DevIndex, byte Chan, byte AoCycle);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoDataFifo(int DevIndex, byte Chan, float[] Voltage, uint Len);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAoFifo(int DevIndex, byte Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSync(int DevIndex, byte Chans);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoImmediately(int DevIndex, byte Chan, float Voltage);

        //--------------------------------------------------------------------------
        // Trig Control

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiSoftTrig(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiSoftTrig(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoSoftTrig(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSoftTrig(int DevIndex, byte Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3GlobalSoftTrig(int DevIndex);

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAiTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDiTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDoTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAoTrigger(int DevIndex, byte Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrGlobalSoftTrig(int DevIndex);

        //--------------------------------------------------------------------------
        // Sync Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ExtTrigOutSource(int DevIndex, byte Source);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ExtConvOutSource(int DevIndex, byte Source);

        //--------------------------------------------------------------------------
        // Get Data Acquired

        [DllImport("USB3000.dll")]
        public static extern int USB3GetAi(int DevIndex, uint Points, float[] Ai, int TimeOut);
        [DllImport("USB3000.dll")]
        public static extern int USB3GetDi(int DevIndex, uint Points, byte[] Di, int TimeOut);
    }
}
