﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;   //必须添加，不然DllImport报错
using System.Threading;
namespace ao_waveform_sync
{
    class Program
    {
        static void Main(string[] args)
        {
            int temp;
	    int i;
	    int DevIndex = 0;
	    byte Chan = 0;
	    float[] waveform = new float[1000];

	    // 生成一个由 1000 个点组成的正弦波
	    for(i = 0; i < 1000; i++)
	    {
		    waveform[i] = waveform[i] = (float)(Math.Sin((i* 2 * Math.PI)/1000));
	    }

	    // 调用函数时，返回错误代码
	    // 打开设备
	    temp = USB3000DLL.USB3OpenDevice(DevIndex);

        //返回值以及代表SN码 以及型号信息
        temp = USB3000DLL.USB3ReadDevcieSN(DevIndex);
        Console.WriteLine("SN码：" + temp);
        temp = USB3000DLL.USB3ReadDevcieModel(DevIndex);
        Console.WriteLine("型号为USB-" + temp);

	    // 分别对 4 个通道进行设置
	    for(Chan = 0; Chan < 4; Chan++)
	    {
		    // 清除触发标志
		    temp =USB3000DLL.SetUSB3ClrAoTrigger(DevIndex, Chan);

		    // 清空 AoFifo
		    temp = USB3000DLL.SetUSB3ClrAoFifo(DevIndex, Chan);

		    // 设置为 NCycle 模式，代表硬件 Fifo 缓冲的有限次数/无限循环输出模式，其循环次数由 SetUSB3AoCycle() 函数决定。
		    temp = USB3000DLL.SetUSB3AoSampleMode(DevIndex, Chan, 1);

		    // 设置 Ao 采样周期为 10000ns，即 100kSa/s
		    temp = USB3000DLL.SetUSB3AoSampleRate(DevIndex, Chan, 10000);

		    // 设置触发源为 GlobalSoftTrig 全局软件触发
		    temp = USB3000DLL.SetUSB3AoTrigSource(DevIndex, Chan, 255);

		    //设置采样时钟源为 Ao 内部采样时钟源，该时采样时钟频率由 SetUSB3AoSampleRate() 函数设定。
		    temp = USB3000DLL.SetUSB3AoConvSource(DevIndex, Chan, 0);

		    // 设置无限循环输出，AoCycle 参数设置为 0 即可。
		    temp = USB3000DLL.SetUSB3AoCycle(DevIndex, Chan, 0);

		    // 将波形数组 waveform 的前 500 个点下载到 AoFifo 中，
		    temp = USB3000DLL.SetUSB3AoDataFifo(DevIndex, Chan, waveform, 1000);
	    }

	    // 选择需要通道的 ao 通道。
	    // 0xf 参数的 bit-0 到 bit 3 对应 ao 0 到 ao 3。
	    // 例如，如果只选择 ai 0 和 ai 1，此参数应设置为 0x03。
	    // 依次类推。
	    temp = USB3000DLL.SetUSB3AoSync(DevIndex, 0x0f);

	    // 启动全局软件触发
	    temp = USB3000DLL.SetUSB3GlobalSoftTrig(DevIndex);

        Thread.Sleep(100);
	
	    // 关闭设备
	    temp = USB3000DLL.USB3CloseDevice(DevIndex);

        /*如需了解更多函数参数意义，则需要访问smmacq.cn对应产品系列函数手册进行查询
         *以及了解各类函数返回值负数代表的错误意义            
         */
        }
    }
    class USB3000DLL
    {
        [DllImport("USB3000.dll")]
        public static extern int FindUSB3DAQ();
        [DllImport("USB3000.dll")]
        public static extern int USB3OpenDevice(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int USB3CloseDevice(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int USB3ReadDevcieSN(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int USB3ReadDevcieModel(int DevIndex);

        //--------------------------------------------------------------------------
        // Ananlog Input Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiSampleMode(int DevIndex, byte AiSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiConnectType(int DevIndex, byte AiConnectType);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiRange(int DevIndex, byte Chan, float AiRange);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiChanSel(int DevIndex, byte Chan, byte Sel);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiTrigSource(int DevIndex, byte AiTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiConvSource(int DevIndex, byte AiConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiPreTrigPoints(int DevIndex, uint AiPreTrigPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiOneShotPoints(int DevIndex, uint AiOneShotPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAiFifo(int DevIndex);

        //--------------------------------------------------------------------------
        // Digital I/O Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiSampleMode(int DevIndex, byte DiSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiTrigSource(int DevIndex, byte DiTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiConvSource(int DevIndex, byte DiConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiPreTrigPoints(int DevIndex, uint DiPreTrigPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiOneShotPoints(int DevIndex, uint DiOneShotPoints);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDiFifo(int DevIndex);

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoSampleMode(int DevIndex, byte DoSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoTrigSource(int DevIndex, byte DoTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoConvSource(int DevIndex, byte DoConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoCycle(int DevIndex, uint DoCycle);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoDataFifo(int DevIndex, uint[] Value, uint Len);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDoFifo(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoWaveCtrl(int DevIndex, uint Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoImmediately(int DevIndex, uint Chan, uint Value);

        //--------------------------------------------------------------------------
        // Ananlog Output Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSampleRate(int DevIndex, byte Chan, uint SamplePeriod);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSampleMode(int DevIndex, byte Chan, byte AoSampleMode);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoTrigSource(int DevIndex, byte Chan, byte AoTrigSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoConvSource(int DevIndex, byte Chan, byte AoConvSource);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoCycle(int DevIndex, byte Chan, byte AoCycle);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoDataFifo(int DevIndex, byte Chan, float[] Voltage, uint Len);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAoFifo(int DevIndex, byte Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSync(int DevIndex, byte Chans);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoImmediately(int DevIndex, byte Chan, float Voltage);

        //--------------------------------------------------------------------------
        // Trig Control

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AiSoftTrig(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DiSoftTrig(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3DoSoftTrig(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3AoSoftTrig(int DevIndex, byte Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3GlobalSoftTrig(int DevIndex);

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAiTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDiTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrDoTrigger(int DevIndex);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrAoTrigger(int DevIndex, byte Chan);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ClrGlobalSoftTrig(int DevIndex);

        //--------------------------------------------------------------------------
        // Sync Configuration

        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ExtTrigOutSource(int DevIndex, byte Source);
        [DllImport("USB3000.dll")]
        public static extern int SetUSB3ExtConvOutSource(int DevIndex, byte Source);

        //--------------------------------------------------------------------------
        // Get Data Acquired

        [DllImport("USB3000.dll")]
        public static extern int USB3GetAi(int DevIndex, uint Points, float[] Ai, int TimeOut);
        [DllImport("USB3000.dll")]
        public static extern int USB3GetDi(int DevIndex, uint Points, byte[] Di, int TimeOut);
    }
}
